@@ -19,8 +19,10 @@ from meta import name as appname, version
 from options import debug
 from util_os import islink, readlink, waccess
 from util_str import safe_unicode, strtr
-from worker import Error, Info, get_current_profile_path, show_result_dialog
+from worker import (Error, Info, UnloggedInfo, get_current_profile_path,
+					show_result_dialog)
 import ICCProfile as ICCP
+import colormath
 import config
 import localization as lang
 import madvr
@@ -29,7 +31,9 @@ from worker import UnloggedWarning, check_set_argyll_bin, get_options_from_profi
 from wxwindows import (BaseApp, BaseFrame, ConfirmDialog, FileDrop, InfoDialog,
 					   wx)
 from wxfixes import TempXmlResource
+import floatspin
 import xh_filebrowsebutton
+import xh_floatspin
 import xh_bitmapctrls
 
 from wx import xrc
@@ -43,6 +47,7 @@ class LUT3DFrame(BaseFrame):
 		self.res = TempXmlResource(get_data_path(os.path.join("xrc", 
 															  "3dlut.xrc")))
 		self.res.InsertHandler(xh_filebrowsebutton.FileBrowseButtonWithHistoryXmlHandler())
+		self.res.InsertHandler(xh_floatspin.FloatSpinCtrlXmlHandler())
 		self.res.InsertHandler(xh_bitmapctrls.BitmapButton())
 		self.res.InsertHandler(xh_bitmapctrls.StaticBitmap())
 		if hasattr(wx, "PreFrame"):
@@ -139,10 +144,19 @@ class LUT3DFrame(BaseFrame):
 										   self.lut3d_trc_black_output_offset_ctrl_handler)
 		self.lut3d_trc_black_output_offset_intctrl.Bind(wx.EVT_TEXT,
 											  self.lut3d_trc_black_output_offset_ctrl_handler)
-		self.lut3d_hdr_peak_luminance_ctrl.Bind(wx.EVT_SPINCTRL,
-											self.lut3d_hdr_peak_luminance_handler)
-		self.lut3d_hdr_peak_luminance_ctrl.Bind(wx.EVT_KILL_FOCUS,
+		self.lut3d_hdr_peak_luminance_ctrl.Bind(wx.EVT_TEXT,
 											self.lut3d_hdr_peak_luminance_handler)
+		self.lut3d_hdr_display_ctrl.Bind(wx.EVT_CHOICE, self.lut3d_hdr_display_handler)
+		self.lut3d_hdr_maxcll_ctrl.Bind(wx.EVT_TEXT,
+										self.lut3d_hdr_maxcll_handler)
+		self.lut3d_content_colorspace_ctrl.Bind(wx.EVT_CHOICE,
+												self.lut3d_content_colorspace_handler)
+		for color in ("white", "red", "green", "blue"):
+			for coord in "xy":
+				v = getcfg("3dlut.content.colorspace.%s.%s" % (color, coord))
+				getattr(self, "lut3d_content_colorspace_%s_%s" %
+							  (color, coord)).Bind(floatspin.EVT_FLOATSPIN,
+												   self.lut3d_content_colorspace_xy_handler)
 		self.encoding_input_ctrl.Bind(wx.EVT_CHOICE,
 											self.lut3d_encoding_input_ctrl_handler)
 		self.encoding_output_ctrl.Bind(wx.EVT_CHOICE,
@@ -200,11 +214,28 @@ class LUT3DFrame(BaseFrame):
 			setcfg("3dlut.apply_trc", int(v))
 			setcfg("3dlut.apply_black_offset",
 				   int(self.lut3d_trc_apply_black_offset_ctrl.GetValue()))
+		self.lut3d_hdr_peak_luminance_label.Enable(v)
+		self.lut3d_hdr_peak_luminance_ctrl.Enable(v)
+		self.lut3d_hdr_peak_luminance_ctrl_label.Enable(v)
+		self.lut3d_hdr_maxcll_label.Enable(v)
+		self.lut3d_hdr_maxcll_ctrl.Enable(v)
+		self.lut3d_hdr_maxcll_ctrl_label.Enable(v)
+		self.lut3d_content_colorspace_label.Enable(v)
+		self.lut3d_content_colorspace_ctrl.Enable(v)
+		for color in ("white", "red", "green", "blue"):
+			for coord in "xy":
+				getattr(self, "lut3d_content_colorspace_%s_label" %
+							  color[0]).Enable(v)
+				getattr(self, "lut3d_content_colorspace_%s_%s" %
+							  (color, coord)).Enable(v)
+				getattr(self, "lut3d_content_colorspace_%s_%s_label" %
+							  (color, coord)).Enable(v)
 		self.lut3d_trc_black_output_offset_label.Enable(v)
 		self.lut3d_trc_black_output_offset_ctrl.Enable(v)
 		self.lut3d_trc_black_output_offset_intctrl.Enable(v)
 		self.lut3d_trc_black_output_offset_intctrl_label.Enable(v)
 		self.lut3d_show_input_value_clipping_warning(event)
+		self.lut3d_show_hdr_display_control()
 
 	def lut3d_show_input_value_clipping_warning(self, layout):
 		self.panel.Freeze()
@@ -223,9 +254,23 @@ class LUT3DFrame(BaseFrame):
 		setcfg("3dlut.output.profile.apply_cal",
 			   int(self.lut3d_apply_cal_cb.GetValue()))
 
+	def lut3d_hdr_display_handler(self, event):
+		if (self.lut3d_hdr_display_ctrl.GetSelection() and
+			not getcfg("3dlut.hdr_display")):
+			if not show_result_dialog(UnloggedInfo(lang.getstr("3dlut.format.madVR.hdr.confirm")),
+									  self, confirm=lang.getstr("ok")):
+				self.lut3d_hdr_display_ctrl.SetSelection(0)
+				return
+		self.lut3d_set_option("3dlut.hdr_display",
+							  self.lut3d_hdr_display_ctrl.GetSelection())
+
 	def lut3d_hdr_peak_luminance_handler(self, event):
-		setcfg("3dlut.hdr_peak_luminance",
-			   self.lut3d_hdr_peak_luminance_ctrl.GetValue())
+		self.lut3d_set_option("3dlut.hdr_peak_luminance",
+							  self.lut3d_hdr_peak_luminance_ctrl.GetValue())
+
+	def lut3d_hdr_maxcll_handler(self, event):
+		self.lut3d_set_option("3dlut.hdr_maxcll",
+							  self.lut3d_hdr_maxcll_ctrl.GetValue())
 
 	def lut3d_trc_black_output_offset_ctrl_handler(self, event):
 		if event.GetId() == self.lut3d_trc_black_output_offset_intctrl.GetId():
@@ -265,21 +310,26 @@ class LUT3DFrame(BaseFrame):
 			self.lut3d_set_option("3dlut.trc_gamma", 2.4)
 			self.lut3d_set_option("3dlut.trc_gamma_type", "B")
 			self.lut3d_set_option("3dlut.trc_output_offset", 0.0)
-			setcfg("3dlut.trc", "bt1886")
-			self.lut3d_update_trc_controls()
+			trc = "bt1886"
 		elif self.lut3d_trc_ctrl.GetSelection() == 0:
 			# Pure power gamma 2.2
 			self.lut3d_set_option("3dlut.trc_gamma", 2.2)
 			self.lut3d_set_option("3dlut.trc_gamma_type", "b")
 			self.lut3d_set_option("3dlut.trc_output_offset", 1.0)
-			setcfg("3dlut.trc", "gamma2.2")
-			self.lut3d_update_trc_controls()
+			trc = "gamma2.2"
 		elif self.lut3d_trc_ctrl.GetSelection() == 2:  # SMPTE 2084, hard clip
-			setcfg("3dlut.trc", "smpte2084.hardclip")
+			self.lut3d_set_option("3dlut.trc_output_offset", 0.0)
+			trc = "smpte2084.hardclip"
+			self.lut3d_set_option("3dlut.hdr_maxcll", 10000)
 		elif self.lut3d_trc_ctrl.GetSelection() == 3:  # SMPTE 2084, roll-off clip
-			setcfg("3dlut.trc", "smpte2084.rolloffclip")
+			self.lut3d_set_option("3dlut.trc_output_offset", 0.0)
+			trc = "smpte2084.rolloffclip"
+			self.lut3d_set_option("3dlut.hdr_maxcll", 10000)
 		else:
-			setcfg("3dlut.trc", "customgamma")
+			trc = "customgamma"
+		self.lut3d_set_option("3dlut.trc", trc)
+		if trc != "customgamma":
+			self.lut3d_update_trc_controls()
 		self.lut3d_show_trc_controls()
 		self.Thaw()
 
@@ -376,6 +426,39 @@ class LUT3DFrame(BaseFrame):
 			self.lut3dframe.lut3d_update_shared_controls()
 		elif self.Parent:
 			self.Parent.lut3d_update_shared_controls()
+
+	def lut3d_content_colorspace_handler(self, event):
+		sel = self.lut3d_content_colorspace_ctrl.Selection
+		try:
+			rgb_space = self.lut3d_content_colorspace_names[sel]
+		except IndexError:
+			# Custom
+			rgb_space = None
+		else:
+			rgb_space = colormath.get_rgb_space(rgb_space)
+			for i, color in enumerate(("white", "red", "green", "blue")):
+				if i == 0:
+					xyY = colormath.XYZ2xyY(*rgb_space[1])
+				else:
+					xyY = rgb_space[2:][i - 1]
+				for j, coord in enumerate("xy"):
+					v = round(xyY[j], 4)
+					self.lut3d_set_option("3dlut.content.colorspace.%s.%s" %
+										  (color, coord), v)
+			self.lut3d_update_trc_controls()
+		self.panel.Freeze()
+		sizer = self.lut3d_content_colorspace_red_x.ContainingSizer
+		sizer.ShowItems(not rgb_space)
+		self.panel.Layout()
+		self.panel.Thaw()
+		if isinstance(self, LUT3DFrame):
+			self.update_layout()
+
+	def lut3d_content_colorspace_xy_handler(self, event):
+		option = event.GetEventObject().Name.replace("_", ".")[5:]
+		self.lut3d_set_option("3dlut" + option,
+							  event.GetEventObject().GetValue())
+		self.lut3d_update_trc_controls()
 	
 	def lut3d_create_consumer(self, result=None):
 		if isinstance(result, Exception):
@@ -445,9 +528,9 @@ class LUT3DFrame(BaseFrame):
 			(profile_in and profile_in.profileClass == "link")):
 			if profile_out and profile_in.isSame(profile_out,
 												 force_calculation=True):
-				show_result_dialog(Error(lang.getstr("error.source_dest_same")),
-								   self)
-				return
+				if not show_result_dialog(Warning(lang.getstr("error.source_dest_same")),
+										  self, confirm=lang.getstr("continue")):
+					return
 			checkoverwrite = True
 			remember_last_3dlut_path = False
 			if not path:
@@ -658,8 +741,11 @@ class LUT3DFrame(BaseFrame):
 														"ColorLookupTable.fx")
 							with open(clut_fx_path, "wb") as clut_fx_file:
 								clut_fx_file.write(clut_fx)
-							with open(reshade_fx_path, "wb") as reshade_fx_file:
-								reshade_fx_file.write(reshade_fx)
+							if not os.path.isdir(os.path.join(dst_dir,
+															  "reshade-shaders")):
+								# ReShade < 3.0
+								with open(reshade_fx_path, "wb") as reshade_fx_file:
+									reshade_fx_file.write(reshade_fx)
 					for src_path, dst_path in zip(src_paths, dst_paths):
 						shutil.copyfile(src_path, dst_path)
 					return
@@ -687,6 +773,12 @@ class LUT3DFrame(BaseFrame):
 				trc_gamma = getcfg("3dlut.trc_gamma")
 		else:
 			trc_gamma = None
+		XYZwp = None
+		if not isinstance(self, LUT3DFrame):
+			x = getcfg("3dlut.whitepoint.x", False)
+			y = getcfg("3dlut.whitepoint.y", False)
+			if x and y:
+				XYZwp = colormath.xyY2XYZ(x, y)
 		trc_gamma_type = getcfg("3dlut.trc_gamma_type")
 		outoffset = getcfg("3dlut.trc_output_offset")
 		intent = getcfg("3dlut.rendering_intent")
@@ -697,6 +789,16 @@ class LUT3DFrame(BaseFrame):
 		apply_black_offset = getcfg("3dlut.apply_black_offset")
 		use_b2a = getcfg("3dlut.gamap.use_b2a")
 		white_cdm2 = getcfg("3dlut.hdr_peak_luminance")
+		maxcll = getcfg("3dlut.hdr_maxcll")
+		content_rgb_space = [-2084, [], [], [], []]
+		for i, color in enumerate(("white", "red", "green", "blue")):
+			for coord in "xy":
+				v = getcfg("3dlut.content.colorspace.%s.%s" % (color, coord))
+				content_rgb_space[i + 1].append(v)
+			# Dummy Y value, not used for primaries but needs to be present
+			content_rgb_space[i + 1].append(1.0)
+		content_rgb_space[1] = colormath.xyY2XYZ(*content_rgb_space[1])
+		content_rgb_space = colormath.get_rgb_space(content_rgb_space)
 		try:
 			self.worker.create_3dlut(profile_in, path, profile_abst,
 									 profile_out, apply_cal=apply_cal,
@@ -709,7 +811,11 @@ class LUT3DFrame(BaseFrame):
 									 trc_gamma_type=trc_gamma_type,
 									 trc_output_offset=outoffset,
 									 apply_black_offset=apply_black_offset,
-									 use_b2a=use_b2a, white_cdm2=white_cdm2)
+									 use_b2a=use_b2a, white_cdm2=white_cdm2,
+									 maxcll=maxcll,
+									 content_rgb_space=content_rgb_space,
+									 hdr_display=getcfg("3dlut.hdr_display"),
+									 XYZwp=XYZwp)
 		except Exception, exception:
 			return exception
 		return True
@@ -782,8 +888,14 @@ class LUT3DFrame(BaseFrame):
 			if getattr(self, "lut3dframe", None):
 				self.lut3dframe.lut3d_update_shared_controls()
 			return
-		elif self.Parent:
-			self.Parent.lut3d_update_shared_controls()
+		else:
+			self.panel.Freeze()
+			self.lut3d_show_hdr_display_control()
+			self.panel.Layout()
+			self.panel.Thaw()
+			self.update_layout()
+			if self.Parent:
+				self.Parent.lut3d_update_shared_controls()
 		self.lut3d_create_btn.Enable(format != "madVR" or
 									 self.output_profile_ctrl.IsShown())
 	
@@ -1074,6 +1186,10 @@ class LUT3DFrame(BaseFrame):
 		self.lut3d_trc_gamma_type_ctrl.SetItems([lang.getstr("trc.type.relative"),
 											  lang.getstr("trc.type.absolute")])
 
+		self.lut3d_content_colorspace_names = ["Rec. 2020", "DCI P3", "Rec. 709"]
+		self.lut3d_content_colorspace_ctrl.SetItems(self.lut3d_content_colorspace_names +
+													[lang.getstr("custom")])
+
 		self.rendering_intents_ab = {}
 		self.rendering_intents_ba = {}
 		self.lut3d_rendering_intent_ctrl.Clear()
@@ -1095,6 +1211,11 @@ class LUT3DFrame(BaseFrame):
 				self.lut3d_formats_ab[i] = format
 				self.lut3d_formats_ba[format] = i
 				i += 1
+
+		self.lut3d_hdr_display_ctrl.SetItems([lang.getstr(item)
+											  for item in
+											  ("3dlut.format.madVR.hdr_to_sdr",
+											   "3dlut.format.madVR.hdr")])
 		
 		self.lut3d_size_ab = {}
 		self.lut3d_size_ba = {}
@@ -1193,6 +1314,7 @@ class LUT3DFrame(BaseFrame):
 			format = "cube"
 			setcfg("3dlut.format", format)
 		self.lut3d_format_ctrl.SetSelection(self.lut3d_formats_ba[getcfg("3dlut.format")])
+		self.lut3d_hdr_display_ctrl.SetSelection(getcfg("3dlut.hdr_display"))
 		self.lut3d_size_ctrl.SetSelection(self.lut3d_size_ba[getcfg("3dlut.size")])
 		self.lut3d_enable_size_controls()
 		self.lut3d_bitdepth_input_ctrl.SetSelection(self.lut3d_bitdepth_ba[getcfg("3dlut.bitdepth.input")])
@@ -1212,12 +1334,15 @@ class LUT3DFrame(BaseFrame):
 			getcfg("3dlut.trc_output_offset") == 0 and
 			getcfg("3dlut.trc_gamma") == 2.4):
 			self.lut3d_trc_ctrl.SetSelection(1)  # BT.1886
+			setcfg("3dlut.trc", "bt1886")
 		elif (getcfg("3dlut.trc_gamma_type") == "b" and
 			getcfg("3dlut.trc_output_offset") == 1 and
 			getcfg("3dlut.trc_gamma") == 2.2):
 			self.lut3d_trc_ctrl.SetSelection(0)  # Pure power gamma 2.2
+			setcfg("3dlut.trc", "gamma2.2")
 		else:
 			self.lut3d_trc_ctrl.SetSelection(4)  # Custom
+			setcfg("3dlut.trc", "customgamma")
 
 	def lut3d_update_trc_controls(self):
 		self.lut3d_update_trc_control()
@@ -1227,6 +1352,28 @@ class LUT3DFrame(BaseFrame):
 		self.lut3d_trc_black_output_offset_ctrl.SetValue(outoffset)
 		self.lut3d_trc_black_output_offset_intctrl.SetValue(outoffset)
 		self.lut3d_hdr_peak_luminance_ctrl.SetValue(getcfg("3dlut.hdr_peak_luminance"))
+		self.lut3d_hdr_maxcll_ctrl.SetValue(getcfg("3dlut.hdr_maxcll"))
+		# Content colorspace (currently only used for SMPTE 2084)
+		content_colors = []
+		for color in ("white", "red", "green", "blue"):
+			for coord in "xy":
+				v = getcfg("3dlut.content.colorspace.%s.%s" % (color, coord))
+				getattr(self, "lut3d_content_colorspace_%s_%s" %
+							  (color, coord)).SetValue(v)
+				content_colors.append(round(v, 4))
+		for i, rgb_space in enumerate(self.lut3d_content_colorspace_names):
+			rgb_space = colormath.get_rgb_space(rgb_space)
+			wx, wy = (round(v, 4) for v in
+					  colormath.XYZ2xyY(*colormath.get_whitepoint(rgb_space[1]))[:2])
+			colors = [wx, wy]
+			for primary in rgb_space[2:5]:
+				for j in xrange(2):
+					colors.append(round(primary[j], 4))
+			if colors == content_colors:
+				break
+		else:
+			i = self.lut3d_content_colorspace_ctrl.Count - 1
+		self.lut3d_content_colorspace_ctrl.SetSelection(i)
 
 	def update_linking_controls(self):
 		self.gamut_mapping_inverse_a2b.SetValue(
@@ -1275,6 +1422,17 @@ class LUT3DFrame(BaseFrame):
 					   getcfg("3dlut.trc_gamma.backup"))
 				setcfg("3dlut.trc_gamma.backup", None)
 			self.lut3d_update_trc_controls()
+		elif (hasattr(self, "input_profile") and
+			  isinstance(self.input_profile.tags.get("A2B0"),
+						 ICCP.LUT16Type) and
+			  isinstance(self.input_profile.tags.get("A2B1", ICCP.LUT16Type()),
+						 ICCP.LUT16Type) and
+			  isinstance(self.input_profile.tags.get("A2B2", ICCP.LUT16Type()),
+						 ICCP.LUT16Type)):
+			self.lut3d_trc_apply_black_offset_ctrl.Enable()
+		else:
+			self.lut3d_trc_apply_black_offset_ctrl.Disable()
+			setcfg("3dlut.apply_black_offset", 0)
 		if getcfg("3dlut.apply_black_offset"):
 			self.lut3d_trc_apply_black_offset_ctrl.SetValue(True)
 		elif getcfg("3dlut.apply_trc"):
@@ -1301,31 +1459,52 @@ class LUT3DFrame(BaseFrame):
 		if not frozen:
 			self.Thaw()
 
+	def lut3d_show_hdr_display_control(self):
+		self.lut3d_hdr_display_ctrl.Show((getcfg("3dlut.apply_trc") or
+								not hasattr(self, "lut3d_trc_apply_none_ctrl")) and
+							   getcfg("3dlut.trc").startswith("smpte2084") and
+							   getcfg("3dlut.format") == "madVR")
+
 	def lut3d_show_trc_controls(self, show=True):
 		self.panel.Freeze()
 		show = show and self.worker.argyll_version >= [1, 6]
 		if hasattr(self, "lut3d_trc_apply_ctrl"):
 			self.lut3d_trc_apply_ctrl.Show(show)
 		self.lut3d_trc_ctrl.Show(show)
-		show = (show and (getcfg("3dlut.trc") == "customgamma" or
-						  (isinstance(self, LUT3DFrame) or
-						   getcfg("show_advanced_options"))) and
-				not getcfg("3dlut.trc").startswith("smpte2084"))
-		self.lut3d_trc_gamma_label.Show(show)
-		self.lut3d_trc_gamma_ctrl.Show(show)
+		smpte2084 = getcfg("3dlut.trc").startswith("smpte2084")
+		show = show and (getcfg("3dlut.trc") == "customgamma" or
+						 (isinstance(self, LUT3DFrame) or
+						  getcfg("show_advanced_options")))
+		self.lut3d_trc_gamma_label.Show(show and not smpte2084)
+		self.lut3d_trc_gamma_ctrl.Show(show and not smpte2084)
+		smpte2084r = getcfg("3dlut.trc") == "smpte2084.rolloffclip"
+		# Show items in this order so we end up with the correct controls shown
+		showcc = smpte2084r and (isinstance(self, LUT3DFrame) or
+								 getcfg("show_advanced_options"))
+		self.lut3d_content_colorspace_label.ContainingSizer.ShowItems(showcc)
+		sel = self.lut3d_content_colorspace_ctrl.Selection
+		lastsel = self.lut3d_content_colorspace_ctrl.Count - 1
+		sizer = self.lut3d_content_colorspace_red_x.ContainingSizer
+		sizer.ShowItems(showcc and sel == lastsel)
+		self.lut3d_hdr_maxcll_label.Show(show and smpte2084r)
+		self.lut3d_hdr_maxcll_ctrl.Show(show and smpte2084r)
+		self.lut3d_hdr_maxcll_ctrl_label.Show(show and smpte2084r)
+		show = show or smpte2084
 		show = show and ((hasattr(self, "lut3d_create_cb") and
 						  getcfg("3dlut.create")) or self.XYZbpout > [0, 0, 0])
-		self.lut3d_trc_gamma_type_ctrl.Show(show)
+		self.lut3d_trc_gamma_type_ctrl.Show(show and not smpte2084)
 		self.lut3d_trc_black_output_offset_label.Show(show)
 		self.lut3d_trc_black_output_offset_ctrl.Show(show)
 		self.lut3d_trc_black_output_offset_intctrl.Show(show)
 		self.lut3d_trc_black_output_offset_intctrl_label.Show(show)
-		show = getcfg("3dlut.trc").startswith("smpte2084")  # SMPTE 2084
-		self.lut3d_hdr_peak_luminance_label.Show(show)
-		self.lut3d_hdr_peak_luminance_ctrl.Show(show)
-		self.lut3d_hdr_peak_luminance_ctrl_label.Show(show)
+		self.lut3d_hdr_peak_luminance_label.Show(smpte2084)
+		self.lut3d_hdr_peak_luminance_ctrl.Show(smpte2084)
+		self.lut3d_hdr_peak_luminance_ctrl_label.Show(smpte2084)
+		self.lut3d_show_hdr_display_control()
 		self.panel.Layout()
 		self.panel.Thaw()
+		if isinstance(self, LUT3DFrame):
+			self.update_layout()
 	
 	def lut3d_show_encoding_controls(self, show=True):
 		show = show and ((self.worker.argyll_version >= [1, 7] and
